@@ -1,7 +1,7 @@
 #
 # This file is part of Checkbox.
 #
-# Copyright 2008 Canonical Ltd.
+# Copyright 2008-2012 Canonical Ltd.
 #
 # Checkbox is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -57,8 +57,12 @@ STATUS_TO_ANSWER = {
     UNSUPPORTED: SKIP_ANSWER,
     UNTESTED: SKIP_ANSWER}
 
+CONTINUE_ANSWER = 'Continue'
+RERUN_ANSWER = 'Rerun'
+RESTART_ANSWER = 'Restart'
 
-class UserInterface(object):
+
+class UserInterface:
     """Abstract base class for encapsulating the workflow and common code for
        any user interface implementation (like GTK, Qt, or CLI).
 
@@ -67,7 +71,10 @@ class UserInterface(object):
     def __init__(self, title, data_path=None):
         self.title = title
         self.data_path = data_path
+        self.progress = None
+        self.ui_flags = {}
 
+        self.report_url = None
         self.direction = NEXT
         self.gettext_domain = "checkbox"
         gettext.textdomain(self.gettext_domain)
@@ -76,9 +83,12 @@ class UserInterface(object):
         logging.info(text)
         return default
 
-    def show_error(self, text):
+    def show_error(self, primary_text,
+                   secondary_text=None, detailed_text=None):
+        text = filter(None, [primary_text, secondary_text, detailed_text])
+        text = '\n'.join(text)
         logging.error(text)
-        raise StopAllException, "Error: %s" % text
+        raise StopAllException("Error: %s" % text)
 
     def show_progress(self, message, function, *args, **kwargs):
         self.show_progress_start(message)
@@ -107,7 +117,7 @@ class UserInterface(object):
     def show_text(self, text, previous=None, next=None):
         return
 
-    def show_entry(self, text, value, previous=None, next=None):
+    def show_entry(self, text, value, label=None, previous=None, next=None):
         return value
 
     def show_check(self, text, options=[], default=[]):
@@ -116,7 +126,7 @@ class UserInterface(object):
     def show_radio(self, text, options=[], default=None):
         return default
 
-    def show_tree(self, text, options={}, default={}):
+    def show_tree(self, text, options={}, default={}, deselect_warning=""):
         return default
 
     def show_test(self, test, runner):
@@ -128,25 +138,6 @@ class UserInterface(object):
 
         Display an error dialog if everything fails."""
 
-        (r, w) = os.pipe()
-        if os.fork() > 0:
-            os.close(w)
-            (pid, status) = os.wait()
-            if status:
-                text = _("Unable to start web browser to open %s." % url)
-                message = os.fdopen(r).readline()
-                if message:
-                    text += "\n" + message
-                self.show_error(text)
-            try:
-                os.close(r)
-            except OSError:
-                pass
-            return
-
-        os.setsid()
-        os.close(r)
-
         # If we are called through sudo, determine the real user id and run the
         # browser with it to get the user's web browser settings.
         try:
@@ -165,38 +156,44 @@ class UserInterface(object):
                 if os.getenv("DISPLAY") and \
                         subprocess.call(["pgrep", "-x", "-u", str(uid), "ksmserver"],
                                 stdout=subprocess.PIPE, stderr=subprocess.PIPE) == 0:
-                    subprocess.call(sudo_prefix + ["kfmclient", "openURL", url])
-                    sys.exit(0)
+                    subprocess.Popen(sudo_prefix + ["kfmclient", "openURL", url])
+                    return
             except OSError:
                 pass
 
             # if gnome-session is running, try gnome-open; special-case firefox
             # (and more generally, mozilla browsers) and epiphany to open a new window
-            # with respectively -new-window and --new-window
+            # with respectively -new-window and --new-window; special-case chromium-browser
+            # to allow file:// URLs as needed by the checkbox report.
             try:
                 if os.getenv("DISPLAY") and \
                         subprocess.call(["pgrep", "-x", "-u", str(uid), "gnome-panel|gconfd-2"],
                                 stdout=subprocess.PIPE, stderr=subprocess.PIPE) == 0:
-                    gct = subprocess.Popen(sudo_prefix + ["gconftool", "--get",
-                        "/desktop/gnome/url-handlers/http/command"],
-                        stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-                    if gct.wait() == 0:
-                        preferred_browser = gct.communicate()[0]
+                    from gi.repository import Gio
+
+                    preferred_xml_app = Gio.app_info_get_default_for_type("application/xml",False)
+                    if preferred_xml_app:
+                        preferred_browser = preferred_xml_app.get_executable()
                         browser = re.match("((firefox|seamonkey|flock)[^\s]*)", preferred_browser)
                         if browser:
-                            subprocess.call(sudo_prefix + [browser.group(0), "-new-window", url])
-                            sys.exit(0)
+                            subprocess.Popen(sudo_prefix + [browser.group(0), "-new-window", url])
+                            return
 
                         browser = re.match("(epiphany[^\s]*)", preferred_browser)
                         if browser:
-                            subprocess.call(sudo_prefix + [browser.group(0), "--new-window", url])
-                            sys.exit(0)
+                            subprocess.Popen(sudo_prefix + [browser.group(0), "--new-window", url])
+                            return
+                        
+                        browser = re.match("(chromium-browser[^\s]*)", preferred_browser)
+                        if browser:
+                            subprocess.Popen(sudo_prefix + [browser.group(0), "--allow-file-access-from-files", url])
+                            return
 
-                        subprocess.call(sudo_prefix + [preferred_browser % url], shell=True)
-                        sys.exit(0)
+                        subprocess.Popen(sudo_prefix + [preferred_browser % url], shell=True)
+                        return
 
-                    if subprocess.call(sudo_prefix + ["gnome-open", url]) == 0:
-                        sys.exit(0)
+                    subprocess.Popen(sudo_prefix + ["gnome-open", url])
+                    return
             except OSError:
                 pass
 
@@ -209,8 +206,24 @@ class UserInterface(object):
                 add_variable("HOME", pwd.getpwuid(uid).pw_dir)
 
             webbrowser.open(url, new=True, autoraise=True)
-            sys.exit(0)
+            return
+
+        except Exception as e:
+            pass
+
+    def show_report(self, text, results):
+        """
+        Display a report of all test case results
+        and make it possible to modify them
+        """
+        raise NotImplementedError
+
+    def update_status(self, job):
+        """
+        If implemented, it will be called after each job finishes.
+        The passed Job object can be used to update in-UI status
+        about each job.
+        """
+        pass
+        
 
-        except Exception, e:
-            os.write(w, str(e))
-            sys.exit(1)
